/*
 * #%L
 * Service Locator Client for CXF
 * %%
 * Copyright (C) 2011 Talend Inc.
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */
package org.talend.esb.locator;

import java.io.IOException;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.namespace.QName;

/**
 * This class should be used by Service Locator consumers. It is able to
 * retrieve the list of available endpoints for a given service. Only a random
 * selection strategy to select a single endpoint out of the list uses. When
 * creating an object, you must specify Name of service for which to get the
 * endpoints and the hosts where ZooKeeper started.
 * <p>
 * Usecases:
 * <ul>
 * <li>For a given Service get the list of existing providers from the SL.
 * <li>Select from the list the endpoint to be used.
 * <li>Keep the list of endpoints for subsequent requests.
 * <li>Refresh (reload) the list of endpoints from SL.
 * <li>Returns a proxy of specified service endpoint interface.
 * </ul>
 * 
 */
public class EndpointRetriever {

	private static final Logger LOG = Logger.getLogger(EndpointRetriever.class
			.getName());

	private List<String> endpointsList;
	private ServiceLocator serviceLocator;
	private QName serviceName;

	public void init() throws ServiceLocatorException, InterruptedException ,
			IOException {
		if (serviceName == null)
			throw new NullPointerException("Service name can not be null");
		if (serviceLocator == null)
			throw new NullPointerException("Service locator can not be null");
		LOG.log(Level.INFO, "Creating EndpointRetriever object for "
				+ serviceName.toString() + " service...");

		endpointsList = receiveEndpointsList();
		if (!isEmptyList()) {
			if (LOG.isLoggable(Level.WARNING)) {
				LOG.log(Level.INFO,
						"Endpoint Retriever was initialized successfully.");
			}
		} else if (LOG.isLoggable(Level.WARNING)) {
			LOG.log(Level.WARNING,
					"Endpoint Retriever was initialized with empty list of endpoints.");
		}
	}

	public boolean isEmptyList() {
		return (endpointsList == null) || endpointsList.isEmpty();
	}

	public List<String> getEndpointsList() {
		if (LOG.isLoggable(Level.FINE)) {
			LOG.log(Level.FINE,
					"List of endpoints: " + endpointsList.toString());
		}
		return endpointsList;
	}

	/**
	 * List of endpoins replaced by new data from the
	 * {@link org.talend.esb.locator.ServiceLocator ServiceLocator}.
	 */
	public void refreshEndpointsList() {
		try {
			endpointsList = receiveEndpointsList();
			if (endpointsList == null) {
				LOG.log(Level.SEVERE, "Cannot receive list of endpoints");
			}
		} catch (ServiceLocatorException e) {
			if (LOG.isLoggable(Level.SEVERE)) {
				LOG.log(Level.SEVERE,
						"Cannot refresh list of endpoints due to ServiceLocatorException", e);
			}
		} catch (InterruptedException e) {
			if (LOG.isLoggable(Level.SEVERE)) {
				LOG.log(Level.SEVERE,
						"Cannot refresh list of endpoints due to InterruptedException", e);
			}
		} catch (IOException e) {
			if (LOG.isLoggable(Level.SEVERE)) {
				LOG.log(Level.SEVERE,
						"Cannot refresh list of endpoints due to IOException", e);
			}
		}
	}

	protected ServiceLocator createServiceLocator() {
		return new ServiceLocator();
	}

	public void setServiceLocator(ServiceLocator serviceLocator) {
		this.serviceLocator = serviceLocator;
	}

	public void setServiceName(QName serviceName) {
		this.serviceName = serviceName;
	}

	/**
	 * Establish a connection to the Service Locator, using
	 * {@link org.talend.esb.locator.ServiceLocator ServiceLocator} instance.
	 * Lookup endpoints for Service Name, specified specified in
	 * {@link #EndpointResolver(QName, String) constructor}. Disconnects from a
	 * Service Locator server.
	 * 
	 * @return all endpoints that currently registered at the Service Locator
	 *         Service.
	 * @throws ServiceLocatorException
	 * @throws InterruptedException
	 * @throws IOException
	 */
	private List<String> receiveEndpointsList() throws ServiceLocatorException,
			InterruptedException, IOException {
		List<String> endpointsList = null;

		serviceLocator.connect();
		endpointsList = serviceLocator.lookup(this.serviceName);
		serviceLocator.disconnect();
		LOG.log(Level.INFO, "For service " + serviceName + " received list of endpoints: "
				+ endpointsList.toString());
		return endpointsList;
	}
}
